unit allrestdwreg;

interface

uses
  LCLIntf, LCLType, LMessages,
  LResources, FormEditingIntf, PropEdits, ComponentEditors, Classes,
  uRESTDWBase, uRESTDWPoolerDB, uDWDatamodule, uDWMassiveBuffer, uRESTDWServerEvents,
  uRESTDWDriverZEOS,
  uRestDWLazDriver;


Type
 TPoolersList = Class(TStringProperty)
 Public
  Function  GetAttributes  : TPropertyAttributes; Override;
  Procedure GetValues(Proc : TGetStrProc);        Override;
  Procedure Edit;                                 Override;
End;

  TDWClientEventsEditor = class(TComponentEditor)
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): string; override;
    procedure ExecuteVerb(Index: Integer); override;
  end;


Procedure Register;

implementation

{$R allrestdwreg.res}

Function TPoolersList.GetAttributes : TPropertyAttributes;
Begin
  // editor, sorted list, multiple selection
 Result := [paValueList, paSortList];
End;

procedure TPoolersList.Edit;
Var
 vTempData : String;
Begin
 Inherited Edit;
 Try
  vTempData := GetValue;
  SetValue(vTempData);
 Finally
 End;
end;

Procedure TPoolersList.GetValues(Proc : TGetStrProc);
Var
 vLista : TStringList;
 I      : Integer;
Begin
 //Provide a list of Poolers
 vLista := Nil;
 With GetComponent(0) as TRESTDWDataBase Do
  Begin
   Try
    vLista := TRESTDWDataBase(GetComponent(0)).GetRestPoolers;
    For I := 0 To vLista.Count -1 Do
     Proc (vLista[I]);
   Except
   End;
   If vLista <> Nil Then
    vLista.Free;
  End;
End;

Procedure Register;
Begin
 {$IFNDEF FPC}
  RegisterNoIcon([TServerMethodDataModule]);
  RegisterCustomModule(TServerMethodDataModule, TCustomModule); //TDataModuleDesignerCustomModule);
 {$ELSE}
  FormEditingHook.RegisterDesignerBaseClass(TServerMethodDataModule);
 {$ENDIF}
 RegisterComponents('RESTDataware-Service',     [TRESTServicePooler,
                                                 TDWServerEvents,
                                                 TRESTServiceCGI,
                                                 TDWClientEvents,
                                                 TRESTClientPooler]);

 RegisterComponents('RESTDataware-DB',         [TRESTDWPoolerDB,
                                                TRESTDWDataBase,
                                                TRESTDWClientSQL,
                                                TDWMassiveSQLCache,
                                                TRESTDWStoredProc,
                                                TRESTDWPoolerList,
                                                TDWMassiveCache]);

 RegisterComponents('RESTDataware-Drivers',   [TRESTDWLazDriver,
                                               TRESTDWDriverZeos]);

 RegisterPropertyEditor(TypeInfo(String), TRESTDWDataBase, 'PoolerName', TPoolersList);
 RegisterComponentEditor(TDWClientEvents, TDWClientEventsEditor);
End;

{ TDWClientEventsEditor }

procedure TDWClientEventsEditor.ExecuteVerb(Index: Integer);
begin
  inherited;
  case Index of
    0: (Component as TDWClientEvents).GetEvents:= True; //chama o GetEvents
    1: (Component as TDWClientEvents).ClearEvents; //chama o GetEvents
  end;
end;

function TDWClientEventsEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '&Get Server Events';
    1: Result := '&Clear Client Events';
  end;
end;

function TDWClientEventsEditor.GetVerbCount: Integer;
begin
  Result := 2;
end;


end.
